// speedtest.cc,  to test performances of forint calculation modes
// 

#include <iostream>
#include <math.h>
#include <stdio.h>
#include <sys/times.h>
#include <time.h>
struct tms;

#include <forint/forint.hh>

using namespace FInt;

double alpha=1., beta=1.2, var1=2, var2=0;

main()
{
  mssg c;
  forint fr(c);

  fr.putVar(alpha,"alpha");
  fr.putVar(beta,"beta");
  fr.putVar(var1,"var1");
  fr.putVar(var2,"var2");

// test expression
  char* expr = "(sin(alpha) > 0.5 && beta <= 1)
      ? var1 + sqrt(var2/10)
      : var2 / M_2_SQRTPI";
  char* expr2 = "x=-1; y=0; z=1e-6; while(x<2) y+=(sin(x)+sin(x+=z))/2*z; y";

  cout << "\nProgram to test performances of forint "
    "calculation modes:\n\n  RVM_MODE, SVM_MODE, DLF_MODE.\n" << endl;
  for (int frml = 1; frml<=2; frml++) {
    if (frml == 1) {
      cout << "___________________________\n"
	   << "First testing formula:\n\n"
	   << "  \'" << expr << "\'\n\n" << endl;
    }
    else {
      cout << "\n___________________________\n"
	   << "Second testing formula:\n\n"
	   << "  \'" << expr2 << "\'\n" << endl;
    }
    cout << "Press <Enter> to start (you have to wait for a DLF generating)..."
	 << flush;
    getchar();

    cout << "\nParsing..." << flush;
    if (frml == 1) {
      if (fr.parse(expr)) {
	cerr << endl << "parse(), error exiting\n";
	exit(1);
      }
    }
    else
      if (fr.parse(expr2)) {
	cerr << endl << "parse(), error exiting\n";
	exit(1);
      }
    cout << "done" << endl;

    cout << "Generating of a dynamically linked function..." << flush;
    if (fr.makeDLF()) {
      cerr << endl << "makeDLF(), error exiting\n";
      getchar();
      exit(1);
    }
    cout << "done" << endl;

    volatile double value = 0;

    double lim = 1;
    if (frml == 1) {
// define CPU performance
      long bt = clock();
      long f=0;
      for (int k=0; k<10000000; k++)
	f += 3;
      long et = clock();
      lim = 100000. * long(CLOCKS_PER_SEC / (et-bt) * 5);
      cout << "\nA value is calculated by the formula above "
	   << lim << " times in each mode.\n"
	   << "Press <Enter> to continue..." << flush;
    }
    else {
      cout << "\nA value is calculated by the formula above "
	"one time in each mode.\nPress <Enter> to continue..." << flush;
    }
    getchar();
    cout << "\nPlease, wait a minute." << endl;

    tms t1, t2;
    times(&t1);
    cout << " Calculation in the RVM mode..." << flush;
    for (int i=0; i<long(lim); i++) {
      var2 = var1 = i;
      value = fr.calculate(RVM_MODE);
    }
    times(&t2);
    double calc_time = (1. * t2.tms_utime - t1.tms_utime)/CLK_TCK;
    cout << "done,       user time:" << ((calc_time<10)?"  ":" ")
	 << calc_time
	 << " seconds." << endl
	 << " Calculation in the SVM mode..." << flush;
    for (int i=0; i<long(lim); i++) {
      var2 = var1 = i;
      value = fr.calculate(SVM_MODE);
    }
    times(&t1);
    calc_time = (1. * t1.tms_utime - t2.tms_utime)/CLK_TCK;
    cout << "done,       user time:" << ((calc_time<10)?"  ":" ")
	 << calc_time
	 << " seconds." << endl
	 << " Calculation in the DLF mode..." << flush;
    for (int i=0; i<long(lim); i++) {
      var2 = var1 = i;
      value = fr.calculate(DLF_MODE);
    }
    times(&t2);
    calc_time = (1. * t2.tms_utime - t1.tms_utime)/CLK_TCK;
    cout << "done,       user time:" << ((calc_time<10)?"  ":" ")
	 << calc_time
	 << " seconds." << endl
	 << " Calculation by a built-in formula..." << flush;
    if (frml == 1)
      for (int i=0; i<long(lim); i++) {
	var2 = var1 = i;
	value = (sin(alpha) > 0.5 && beta <= 1)
	  ? var1 + sqrt(var2/10.0)
	  : var2 / M_2_SQRTPI;
      }
    else {
      fr.x()=-1;
      fr.y()=0;
      fr.z()=1e-6;
      while(fr.x()<2)
	fr.y()+=(sin(fr.x())+sin(fr.x()+=fr.z()))/2*fr.z();
      value=fr.y();
    }
    times(&t1);
    calc_time = (1. * t1.tms_utime - t2.tms_utime)/CLK_TCK;
    cout << "done, user time:" << ((calc_time<10)?"  ":" ")
	 << calc_time
	 << " seconds." << endl;
  }
}
